package ink.wgink.gateway.filter.wg;

import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import org.springframework.web.server.WebFilter;
import org.springframework.web.server.WebFilterChain;
import org.springframework.web.server.WebSession;
import org.springframework.web.util.pattern.PathPattern;
import org.springframework.web.util.pattern.PathPatternParser;
import reactor.core.publisher.Mono;

import java.net.URI;
import java.util.Arrays;
import java.util.List;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: WebFluxConfig
 * @Description:
 * @Author: wanggeng
 * @Date: 2021/4/14 2:42 下午
 * @Version: 1.0
 */
@Component
public class WebFluxFilter implements WebFilter {

    public static final String SESSION_USER = "SESSION_USER";
    public static final String PAGE_LOGIN = "/wg/login.html";

    private List<PathPattern> passPathPatterns = Arrays.asList(new PathPattern[]{
            new PathPatternParser().parse("/gw/**"),
            new PathPatternParser().parse("/wg/assets/**"),
            new PathPatternParser().parse("/wg/api/sign/**"),
            new PathPatternParser().parse("/wg/login.html"),
            new PathPatternParser().parse(PAGE_LOGIN),
    });

    @Override
    public Mono<Void> filter(ServerWebExchange serverWebExchange, WebFilterChain webFilterChain) {
        if (this.isPass(serverWebExchange)) {
            return webFilterChain.filter(serverWebExchange);
        }
        return serverWebExchange.getSession().flatMap(webSession -> {
            if (this.isLogin(webSession)) {
                return webFilterChain.filter(serverWebExchange);
            }
            serverWebExchange.getResponse().setStatusCode(HttpStatus.FOUND);
            HttpHeaders headers = serverWebExchange.getResponse().getHeaders();
            headers.setLocation(URI.create(PAGE_LOGIN));
            return serverWebExchange.getResponse().setComplete();
        });
    }

    /**
     * 是否不校验登录
     *
     * @param serverWebExchange
     * @return
     */
    private boolean isPass(ServerWebExchange serverWebExchange) {
        return passPathPatterns.stream().anyMatch(pathPattern -> pathPattern.matches(serverWebExchange.getRequest().getPath()));
    }

    /**
     * 是否登录
     *
     * @param webSession
     * @return
     */
    private boolean isLogin(WebSession webSession) {
        return webSession.getAttributes().get(SESSION_USER) != null;
    }

}
